@@ -25,6 +25,27 @@
 #include <QQmlEngine>
 #include <QDebug>
 
+DelegateRecyclerAttached::DelegateRecyclerAttached(QObject *parent)
+    : QObject(parent)
+{
+}
+
+DelegateRecyclerAttached::~DelegateRecyclerAttached()
+{}
+/*
+void setRecycler(DelegateRecycler *recycler)
+{
+    m_recycler = recycler;
+}
+
+DelegateRecycler *recycler() const
+{
+    return m_recycler;
+}
+*/
+
+
+
 class DelegateCache
 {
 public:
@@ -84,6 +105,11 @@ void DelegateCache::insert(QQmlComponent *component, QQuickItem *item)
         return;
     }
 
+    DelegateRecyclerAttached *attached = qobject_cast<DelegateRecyclerAttached *>(qmlAttachedPropertiesObject<DelegateRecycler>(item, false));
+    if (attached) {
+        emit attached->pooled();
+    }
+    
     item->setParentItem(nullptr);
     items.append(item);
 }
@@ -104,6 +130,7 @@ QQuickItem *DelegateCache::take(QQmlComponent *component)
 DelegateRecycler::DelegateRecycler(QQuickItem *parent)
     : QQuickItem(parent)
 {
+    setFlags(QQuickItem::ItemIsFocusScope);
 }
 
 DelegateRecycler::~DelegateRecycler()
@@ -143,6 +170,24 @@ void DelegateRecycler::syncModel()
     }
 }
 
+void DelegateRecycler::syncModelProperties()
+{
+    const QVariant model = m_propertiesTracker->property("trackedModel");
+    if (!model.isValid()) {
+        return;
+    }
+    QQmlContext *ctx = QQmlEngine::contextForObject(m_item)->parentContext();
+
+    //try to bind all properties
+    QObject *modelObj = model.value<QObject *>();
+    if (modelObj) {
+        const QMetaObject *metaObj = modelObj->metaObject();
+        for (int i = metaObj->propertyOffset(); i < metaObj->propertyCount(); ++i) {
+            ctx->setContextProperty(QString::fromUtf8(metaObj->property(i).name()), metaObj->property(i).read(modelObj));
+        }
+    }
+}
+
 void DelegateRecycler::syncModelData()
 {
     const QVariant newModelData = m_propertiesTracker->property("trackedModelData");
@@ -176,7 +221,7 @@ void DelegateRecycler::setSourceComponent(QQmlComponent *component)
             connect(engine, &QObject::destroyed, engine, [engine] { propertiesTrackerComponent.remove(engine); });
             it = propertiesTrackerComponent.insert(engine, new QQmlComponent(engine, engine));
 
-            (*it)->setData(QByteArrayLiteral("import QtQuick 2.3\nQtObject{property int trackedIndex: index; property var trackedModel: typeof model != 'undefined' ? model : null; property var trackedModelData: typeof modelData != 'undefined' ? modelData : null}"), QUrl());
+            (*it)->setData(QByteArrayLiteral("import QtQuick 2.3\nQtObject{property int trackedIndex: index; property var trackedModel: typeof model != 'undefined' ? model : null; property var trackedModelData: typeof modelData != 'undefined' ? modelData : null}"), QUrl(QStringLiteral("delegaterecycler.cpp")));
         }
         m_propertiesTracker = (*it)->create(QQmlEngine::contextForObject(this));
 
@@ -235,9 +280,16 @@ void DelegateRecycler::setSourceComponent(QQmlComponent *component)
         if (modelObj) {
             const QMetaObject *metaObj = modelObj->metaObject();
             for (int i = metaObj->propertyOffset(); i < metaObj->propertyCount(); ++i) {
-                ctx->setContextProperty(QString::fromUtf8(metaObj->property(i).name()), metaObj->property(i).read(modelObj));
+                QMetaProperty prop = metaObj->property(i);
+                ctx->setContextProperty(QString::fromUtf8(prop.name()), prop.read(modelObj));
+                if (prop.hasNotifySignal()) {
+                    QMetaMethod updateSlot = metaObject()->method(
+                    metaObject()->indexOfSlot("syncModelProperties()"));
+                    connect(modelObj, prop.notifySignal(), this, updateSlot);
+                }
             }
         }
+
         ctx->setContextProperty(QStringLiteral("model"), m_propertiesTracker->property("trackedModel"));
         ctx->setContextProperty(QStringLiteral("modelData"), m_propertiesTracker->property("trackedModelData"));
         ctx->setContextProperty(QStringLiteral("index"), m_propertiesTracker->property("trackedIndex"));
@@ -263,16 +315,15 @@ void DelegateRecycler::setSourceComponent(QQmlComponent *component)
         syncModel();
 
         QQmlContext *ctx = QQmlEngine::contextForObject(m_item)->parentContext();
-#if QT_VERSION < QT_VERSION_CHECK(5, 11, 0)
-        ctx->setContextProperty(QStringLiteral("modelData"), m_propertiesTracker->property("trackedModelData"));
-        ctx->setContextProperty(QStringLiteral("index"), m_propertiesTracker->property("trackedIndex"));
-        ctx->setContextProperty(QStringLiteral("delegateRecycler"), QVariant::fromValue<QObject*>(this));
-#else
         ctx->setContextProperties({ QQmlContext::PropertyPair{ QStringLiteral("modelData"), m_propertiesTracker->property("trackedModelData") },
                                     QQmlContext::PropertyPair{ QStringLiteral("index"), m_propertiesTracker->property("trackedIndex")},
                                     QQmlContext::PropertyPair{ QStringLiteral("delegateRecycler"), QVariant::fromValue<QObject*>(this) }
                                  });
-#endif
+
+        DelegateRecyclerAttached *attached = qobject_cast<DelegateRecyclerAttached *>(qmlAttachedPropertiesObject<DelegateRecycler>(m_item, false));
+        if (attached) {
+            emit attached->reused();
+        }
     }
 
     if (m_item) {
@@ -292,6 +343,11 @@ void DelegateRecycler::resetSourceComponent()
     m_sourceComponent = nullptr;
 }
 
+DelegateRecyclerAttached *DelegateRecycler::qmlAttachedProperties(QObject *object)
+{
+    return new DelegateRecyclerAttached(object);
+}
+
 void DelegateRecycler::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     if (m_item && newGeometry.size() != oldGeometry.size()) {
@@ -300,6 +356,16 @@ void DelegateRecycler::geometryChanged(const QRectF &newGeometry, const QRectF &
     QQuickItem::geometryChanged(newGeometry, oldGeometry);
 }
 
+void DelegateRecycler::focusInEvent(QFocusEvent *event)
+{
+    QQuickItem::focusInEvent(event);
+    if (!m_item) {
+        return;
+    }
+
+    m_item->setFocus(event->reason());
+}
+
 void DelegateRecycler::updateHints()
 {
     updateSize(false);
